﻿using System;
using System.Linq;
using System.Security.Claims;
using System.Threading;
using System.Threading.Tasks;
using InternshipSystem.Api.Options;
using InternshipSystem.Api.Security;
using InternshipSystem.Repository;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Options;

namespace InternshipSystem.Api.Controllers
{
    [Route("access")]
    [ApiController]
    public class AccessController : ControllerBase
    {
        private readonly InternshipDbContext _context;
        private readonly JwtTokenService _tokenService;
        private readonly SecurityOptions _securityOptions;

        public AccessController(IOptions<SecurityOptions> options, InternshipDbContext context, JwtTokenService tokenService)
        {
            _context = context;
            _tokenService = tokenService;
            _securityOptions = options.Value;
        }
        

        [HttpGet("login")]
        public async Task<ActionResult> Authenticate(string code, CancellationToken cancellationToken)
        {
            var identity = new ClaimsIdentity(new[]
            {
                new Claim(ClaimTypes.Name, "Jan"),
                new Claim(ClaimTypes.Surname, "Kowalski"),
                new Claim(InternshipClaims.PersonNumber, "1") 
            });

            return Ok(_tokenService.generateToken(identity));
        }

        [HttpGet("loginEdition")]
        [Authorize]
        public async Task<ActionResult> LoginIntoEdition(Guid editionId, [FromServices] User user, CancellationToken token)
        {
            var edition = await _context.Editions.FindAsync(editionId);

            var hasInternship = await _context.Entry(edition)
                .Collection(e => e.Internships)
                .Query()
                .AnyAsync(i => i.Student.Id == user.PersonNumber, token);

            if (!hasInternship)
            {
                return Unauthorized("Student isn't registered for this edition");
            }

            var newIdentity = User.Identities.First();
            
            newIdentity.AddClaim(new Claim(InternshipClaims.Edition, editionId.ToString()));

            return Ok(_tokenService.generateToken(newIdentity));
        }
    }
}